@@ -2,7 +2,6 @@ import types
 import time
 from PIL import Image
 from imageutils.size import (
-    scale,
     scale_size,
     aspect_ratio_approximate,
     same_aspect_ratio,
@@ -26,13 +25,17 @@ from yafowil.common import (
 )
 
 
-@managedprops('image', *css_managed_props)
+@managedprops('src', 'alt', *css_managed_props)
 def image_edit_renderer(widget, data):
-    if not widget.attrs['src']:
+    src = widget.attrs['src']
+    if callable(src):
+        src = src(widget, data)
+    if not src:
         return data.rendered
+    src = src + '?nocache=%i' % time.time()
     tag = data.tag
     img_attrs = {
-        'src': widget.attrs['src'] + '?nocache=%i' % time.time(),
+        'src': src,
         'alt': widget.attrs['alt'],
         'id': cssid(widget, 'image-preview'),
         'class': 'image-preview',
@@ -41,6 +44,7 @@ def image_edit_renderer(widget, data):
     return img + data.rendered
 
 
+@managedprops('accept')
 def mimetype_extractor(widget, data):
     """XXX: Move relevant parts to ``yafowil.common.mimetype_extractor``.
     """
@@ -66,9 +70,13 @@ def image_extractor(widget, data):
     if data.extracted and data.extracted['file']:
         data.extracted['image'] = Image.open(data.extracted['file'])
         data.extracted['file'].seek(0)
+    if data.extracted is UNSET:
+        # None means submitted but no upload
+        return None
     return data.extracted
 
 
+@managedprops('minsize', 'maxsize')
 def size_extractor(widget, data):
     minsize = widget.attrs['minsize']
     maxsize = widget.attrs['maxsize']
@@ -94,6 +102,7 @@ def size_extractor(widget, data):
     return data.extracted
 
 
+@managedprops('mindpi', 'maxdpi')
 def dpi_extractor(widget, data):
     mindpi = widget.attrs['mindpi']
     maxdpi = widget.attrs['maxdpi']
@@ -119,19 +128,30 @@ def dpi_extractor(widget, data):
     return data.extracted
 
 
+@managedprops('scales')
 def scales_extractor(widget, data):
     scales = widget.attrs['scales']
     if not scales or not data.extracted or not data.extracted.get('image'):
         return data.extracted
     image = data.extracted['image']
     scaled_images = dict()
+    image_appr = aspect_ratio_approximate(image.size)
     for name, size in scales.items():
-        image_size = scale(size, image.size)
+        scale_appr = aspect_ratio_approximate(size)
+        if same_aspect_ratio(size, image.size):
+            image_size = size
+        # scale x
+        if image_appr > scale_appr:
+            image_size = scale_size(image.size, (size[0], None))
+        # scale y
+        if image_appr < scale_appr:
+            image_size = scale_size(image.size, (None, size[1]))
         scaled_images[name] = image.resize(image_size, Image.ANTIALIAS)
     data.extracted['scales'] = scaled_images
     return data.extracted
 
 
+@managedprops('crop')
 def crop_extractor(widget, data):
     """XXX:
     - support cropping definitions as request parameters.
@@ -170,22 +190,26 @@ def crop_extractor(widget, data):
     return data.extracted
 
 
+@managedprops('src', 'alt')
 def image_display_renderer(widget, data):
-    if widget.attrs['src']:
+    src = widget.attrs['src']
+    if src:
+        if callable(src):
+            src = src(widget, data)
         tag = data.tag
         img_attrs = {
-            'src': widget.attrs['src'],
+            'src': src,
             'alt': widget.attrs['alt'],
         }
         return tag('img', **img_attrs)
-    return data.rendered
+    return ''
 
 
 factory.register(
     'image',
     extractors=[
-        generic_required_extractor,
         file_extractor,
+        generic_required_extractor,
         mimetype_extractor,
         image_extractor,
         size_extractor,
